// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "display/Color.hpp"

namespace Dawn {
  enum TextureFormat {
    TEXTURE_FORMAT_R = 1,
    TEXTURE_FORMAT_RG = 2,
    TEXTURE_FORMAT_RGB = 3,
    TEXTURE_FORMAT_RGBA = 4
  };

  enum TextureWrapMode {
    TEXTURE_WRAP_MODE_REPEAT = 0,
    TEXTURE_WRAP_MODE_MIRRORED_REPEAT = 1,
    TEXTURE_WRAP_MODE_CLAMP_TO_EDGE = 2,
    TEXTURE_WRAP_MODE_CLAMP_TO_BORDER = 3
  };

  enum TextureFilterMode {
    TEXTURE_FILTER_MODE_NEAREST = 0,
    TEXTURE_FILTER_MODE_LINEAR = 1
  };

  enum TextureDataFormat {
    TEXTURE_DATA_FORMAT_UNSIGNED_BYTE = 0,
    TEXTURE_DATA_FORMAT_FLOAT = 1
  };

  class ITexture {
    public:
      enum TextureWrapMode wrapModeX = TEXTURE_WRAP_MODE_REPEAT;
      enum TextureWrapMode wrapModeY = TEXTURE_WRAP_MODE_REPEAT;
      enum TextureFilterMode filterModeMin = TEXTURE_FILTER_MODE_NEAREST;
      enum TextureFilterMode filterModeMag = TEXTURE_FILTER_MODE_NEAREST;
      enum TextureFilterMode mipMapFilterModeMin = TEXTURE_FILTER_MODE_NEAREST;
      enum TextureFilterMode mipMapFilterModeMag = TEXTURE_FILTER_MODE_NEAREST;

      /**
       * Returns the width of the texture.
       * 
       * @return Width of the texture.
       */
      virtual int32_t getWidth() = 0;

      /**
       * Returns the height of the texture.
       * 
       * @return Height of the texture.
       */
      virtual int32_t getHeight() = 0;

      /**
       * Initializes a texture.
       * 
       * @param width Width of the texture (in pixels).
       * @param height Height of the texture (in pixels).
       * @param format Data format of the texture to use.
       * @param dataFormat Data format of the texture to use.
       */
      virtual void setSize(
        const int32_t width,
        const int32_t height,
        const enum TextureFormat format,
        const enum TextureDataFormat dataFormat
      ) = 0;

      /**
       * Returns true only when the texture has been loaded, sized and put on
       * the gpu for rendering.
       * 
       * @return True if ready, otherwise false.
       */
      // virtual bool_t isReady() = 0;

      /**
       * Buffer pixel data onto the GPU. Pixel buffering is rather costly so
       * avoid doing this too often.
       * 
       * @param pixels Array of pixels you're trying to buffer.
       */
      virtual void buffer(const struct ColorU8 pixels[]) = 0;

      /**
       * Buffer pixel data onto the GPU. Pixel buffering is rather costly so
       * avoid doing this too often.
       * 
       * @param pixels Array of pixels you're trying to buffer.
       */
      virtual void buffer(const uint8_t pixels[]) = 0;

      /**
       * Binds the texture to the given slot (for use by the shaders).
       * 
       * @param slot Slot to bind to.
       */
      virtual void bind(const uint8_t slot) = 0;

      /**
       * Disposes of the texture.
       */
      virtual ~ITexture() {
      }
  };
}